// Géolocalisation par GPS
// Copyright Philippe Labbe 2021
// Modifié par F6IDT pour utilisation d'un LCD 2x16 et
// calcul du locator 10 caractères
// Version avec test batterie Li-ion

// Chargement des bibliothèques
#include <Wire.h>
#include <LiquidCrystal.h>
#include <SoftwareSerial.h>
#include <TinyGPS++.h>
#include <maidenhead.h>

// lcd(RS, E, D4, D5, D6, D7)
LiquidCrystal lcd(5, 6, 7, 8, 9, 10);

// Déclaration des variables pour le GPS
char data;
double latitude;
double longitude;
double alt;
unsigned long nbre_sat;

byte degre = B11011111;

#define TENSION_ADC A0     // Mesure tension batterie sur A0
#define MOY_NUM 20         // Nombre de mesures à moyenner
#define TensionAlerte 3.5  // Alerte tension basse batterie
#define TensionMini 3.2    // Tension batterie mini avant recharge

float TensionBat = 0;
unsigned long BattTest = 0;  // Interval de test

// Création de l'objet GPS
TinyGPSPlus gps;

// Déclaration des variables pour la liaison série  entre l'arduino et le GPS
#define RX 2
#define TX 3  // Pas câblé mais déclaration nécessaire

// Création de l'objet GPS pour la liaison entre l'arduino et le GPS
SoftwareSerial GPS(RX, TX);
//********************************************************
void setup()
{
   Serial.begin(9600);  // vitesse moniteur série (si utilisé)

  // Initialisation de l'affichage LCD
  lcd.begin(16, 2);

  // La broche 15 (A1) actionne une LED en cas de batterie faible <3,2V 
  pinMode(15, OUTPUT);
  digitalWrite(15, LOW);

  // Initialisation connexion avec le GPS  
  GPS.begin(9600); // vitesse en bauds du GPS
  
  // Message d'accueil  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" ATTENTE TRAMES ");
  lcd.setCursor(0, 1);
  lcd.print("  NMEA VALIDES  ");
}
//--------------------------------------------------------
///////////// Boucle programme ///////////////////////////
//--------------------------------------------------------
void loop()
{
     while (GPS.available())
    {
    data = GPS.read();
    gps.encode(data);
    if (gps.altitude.isUpdated() && gps.satellites.isUpdated())
    {
      latitude = gps.location.lat();
      longitude = gps.location.lng();
      alt = gps.altitude.meters();
      nbre_sat = gps.satellites.value();
      
//--- DATE et HEURE --------------------------

    lcd.clear();
    lcd.setCursor(3, 0);
    lcd.print(gps.date.day());
    lcd.print("/");
    lcd.print(gps.date.month());
    lcd.print("/");
    lcd.print(gps.date.year());
    
    lcd.setCursor(3, 1);
    if (gps.time.hour() < 10) lcd.print("0");
    lcd.print(gps.time.hour());
    lcd.print(":");
    if (gps.time.minute() < 10) lcd.print("0");
    lcd.print(gps.time.minute());
    lcd.print(":");
    if (gps.time.second() < 10) lcd.print("0");
    lcd.print(gps.time.second()); 
    lcd.setCursor(12, 1);
    lcd.print("TU");
    delay(4000);
    
//---- LATITUDE et LONGITUDE ---------------------
      
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Lat : ");
    lcd.setCursor(6, 0);
    lcd.print(latitude, 6);
    lcd.write(0b11011111);     // symbole degré
    
    lcd.setCursor(0, 1);
    lcd.print("Lon : ");
    lcd.setCursor(6, 1);
    lcd.print(longitude, 6);
    lcd.write(0b11011111);     // symbole degré       
    delay(4000);
    
//---- LOCATOR ------------------------------------
      
    lcd.clear();
    lcd.setCursor(3, 0);
    lcd.print("Locator :");
// Utilisation de la bibliothèque. 10 est le nombre de caractères
// du locator mais peut-être 4, 6 ou 8.
    lcd.setCursor(2, 1);
    lcd.print(get_mh(latitude, longitude, 10)); 
    delay(4000);

//---- ALTITUDE et NOMBRE DE SATELITTES ----------
    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Alt: ");
    lcd.setCursor(6, 0);
    lcd.print(alt, 1);
    lcd.print(" mtrs");
          
    lcd.setCursor(0, 1);
    lcd.print("Sat: ");
    lcd.print(nbre_sat);
    lcd.setCursor(8, 1);
    lcd.print("U= ");
    lcd.print(TensionBat, 2);
    lcd.print("V");    
    delay(4000);  
      
//---- SURVEILLANCE BATTERIE --------------------

    read_tension_bat(); // routine de lecture de l'entrée analogique
    test_tension();     // Surveillance batterie et alarme       
   }   
  } 
}
//-----------------------------------------------------------------
///// ROUTINE DE LECTURE ET MOYENNAGE DE L'ENTREE ANALOGIQUE //////
//-----------------------------------------------------------------
int read_adc(int adc_parameter)
{  
 int sum = 0;
 int sample ;
 for (int i = 0; i < MOY_NUM; i++) 
  {                                      // Boucle MOY_NUM de fois
    sample = analogRead(adc_parameter);  // Lecture de la valeur  
    sum += sample;                       // Somme des mesures avant moyenne
    delayMicroseconds(50);               // Pause de 50 microsecondes  
  }
 return(sum / MOY_NUM);                  // Calcul de la moyenne
}
//-----------------------------------------------------------------
////////// ROUTINE DE LECTURE ENTREE ANALOGIQUE////////////////////
//-----------------------------------------------------------------
void read_tension_bat() 
{
// Pour 5V on a ADC = 1024 d'où 1 ADC = (5/1024)Volt = 0.00488 Volt
// La valeur de 0,00488 est théorique, dépend de la valeur exacte
// du 5 volts et devra éventuellement être ajustée.
  TensionBat = read_adc(TENSION_ADC)*0.00488;
}
//-----------------------------------------------------------------
////////// ROUTINE DE SURVEILLANCE BATTERIE ///////////////////////
//-----------------------------------------------------------------
void test_tension()
{
  if (TensionBat <= TensionAlerte && TensionBat > TensionMini)
  {
   lcd.clear(); 
   lcd.setCursor(0, 0);      
   lcd.print("TENSION BASSE ! ");
   lcd.setCursor(0, 1);
   lcd.print("U = ");
   lcd.print(TensionBat, 2);
   lcd.print(" Volts  ");
   digitalWrite(15, HIGH); // allumage de la diode
   delay(4000);
   digitalWrite(15, LOW);  // extinction de la diode     
  }    
  else if (TensionBat <= TensionMini)
  {
   lcd.clear(); 
   lcd.setCursor(0, 0);      
   lcd.print("** ATTENTION ** ");
   lcd.setCursor(0, 1);      
   lcd.print("LI-ION A CHARGER");
   digitalWrite(15, HIGH); // allumage de la diode
   delay(4000);
   //while(1); // si on désire l'arrêt du programme
   }  
}  
//****************************************************************  
